@@ -452,8 +452,7 @@ void uwsgi_setup_mules_and_farms() {
 			mules_list[0] = 0;
 			mules_list++;
 
-			strncpy(uwsgi.farms[i].name, farm_value, 0xff);
-			
+			snprintf(uwsgi.farms[i].name, 0xff, "%s", farm_value);
 
 			// create the socket pipe
 			create_signal_pipe(uwsgi.farms[i].signal_pipe);
