@@ -679,6 +679,10 @@ static struct uwsgi_buffer *uwsgi_subscription_ub(char *key, size_t keysize, uin
 		goto end;
 	if (uwsgi_buffer_append_keyval(ub, "address", 7, socket_name, strlen(socket_name)))
 		goto end;
+
+	if (uwsgi.subscribe_with_modifier1) {
+		modifier1 = atoi(uwsgi.subscribe_with_modifier1);
+	}
 	if (uwsgi_buffer_append_keynum(ub, "modifier1", 9, modifier1))
 		goto end;
 	if (uwsgi_buffer_append_keynum(ub, "modifier2", 9, modifier2))
@@ -1072,6 +1076,12 @@ end:
 		free(s2_sign);
 	if (s2_check)
 		free(s2_check);
+	if (s2_sni_crt)
+		free(s2_sni_crt);
+	if (s2_sni_key)
+		free(s2_sni_key);
+	if (s2_sni_ca)
+		free(s2_sni_ca);
 }
 
 void uwsgi_subscribe_all(uint8_t cmd, int verbose) {
