@@ -5,9 +5,6 @@ extern struct uwsgi_python up;
 extern PyTypeObject uwsgi_InputType;
 
 
-extern struct http_status_codes hsc[];
-
-
 void *uwsgi_request_subhandler_pump(struct wsgi_request *wsgi_req, struct uwsgi_app *wi) {
 
 	PyObject *zero;
@@ -104,27 +101,12 @@ void *uwsgi_request_subhandler_pump(struct wsgi_request *wsgi_req, struct uwsgi_
 	PyDict_SetItemString(wsgi_req->async_environ, "headers", headers);
 	Py_DECREF(headers);
 
-        // if async_post is mapped as a file, directly use it as wsgi.input
-        if (wsgi_req->async_post) {
-#ifdef PYTHREE
-                wsgi_req->async_input = PyFile_FromFd(fileno((FILE *)wsgi_req->async_post), "pump_body", "rb", 0, NULL, NULL, NULL, 0);
-#else
-                wsgi_req->async_input = PyFile_FromFile(wsgi_req->async_post, "pump_body", "r", NULL);
-#endif
-        }
-        else {
-                // create wsgi.input custom object
-                wsgi_req->async_input = (PyObject *) PyObject_New(uwsgi_Input, &uwsgi_InputType);
-                ((uwsgi_Input*)wsgi_req->async_input)->wsgi_req = wsgi_req;
-                ((uwsgi_Input*)wsgi_req->async_input)->pos = 0;
-                ((uwsgi_Input*)wsgi_req->async_input)->readline_pos = 0;
-                ((uwsgi_Input*)wsgi_req->async_input)->readline_max_size = 0;
-
-        }
+        // create wsgi.input custom object
+        wsgi_req->async_input = (PyObject *) PyObject_New(uwsgi_Input, &uwsgi_InputType);
+        ((uwsgi_Input*)wsgi_req->async_input)->wsgi_req = wsgi_req;
 
         PyDict_SetItemString(wsgi_req->async_environ, "body", wsgi_req->async_input);
 
-
 	if (wsgi_req->scheme_len > 0) {
 		zero = PyString_FromStringAndSize(wsgi_req->scheme, wsgi_req->scheme_len);
 	}
@@ -156,22 +138,16 @@ void *uwsgi_request_subhandler_pump(struct wsgi_request *wsgi_req, struct uwsgi_
                 PyDict_SetItemString(wsgi_req->async_environ, "uwsgi.core", PyInt_FromLong(wsgi_req->async_id));
         }
 
-        // cache this ?
-        if (uwsgi.cluster_fd >= 0) {
-                zero = PyString_FromString(uwsgi.cluster);
-                PyDict_SetItemString(wsgi_req->async_environ, "uwsgi.cluster", zero);
-                Py_DECREF(zero);
-                zero = PyString_FromString(uwsgi.hostname);
-                PyDict_SetItemString(wsgi_req->async_environ, "uwsgi.cluster_node", zero);
-                Py_DECREF(zero);
-        }
-
         PyDict_SetItemString(wsgi_req->async_environ, "uwsgi.node", wi->uwsgi_node);
 
-
 	// call
 
-	PyTuple_SetItem(wsgi_req->async_args, 0, wsgi_req->async_environ);
+	if (PyTuple_GetItem(wsgi_req->async_args, 0) != wsgi_req->async_environ) {
+	    if (PyTuple_SetItem(wsgi_req->async_args, 0, wsgi_req->async_environ)) {
+	        uwsgi_log_verbose("unable to set environ to the python application callable, consider using the holy env allocator\n");
+	        return NULL;
+	    }
+	}
 	return python_call(wsgi_req->async_app, wsgi_req->async_args, uwsgi.catch_exceptions, wsgi_req);
 }
 
@@ -179,10 +155,8 @@ void *uwsgi_request_subhandler_pump(struct wsgi_request *wsgi_req, struct uwsgi_
 int uwsgi_response_subhandler_pump(struct wsgi_request *wsgi_req) {
 
 	PyObject *pychunk;
-	ssize_t wsize;
 	int i;
 
-	struct http_status_codes *http_sc;
 	char sc[4];
 
 	// ok its a yield
@@ -220,40 +194,10 @@ int uwsgi_response_subhandler_pump(struct wsgi_request *wsgi_req) {
 				goto clear; 
 			}
 
-			int found = 0;
-			for (http_sc = hsc; http_sc->message != NULL; http_sc++) {
-                		if (http_sc->key[0] == sc[0] && http_sc->key[1] == sc[1] && http_sc->key[2] == sc[2]) {
-                        		wsgi_req->hvec[4].iov_base = (char *) http_sc->message;
-                        		wsgi_req->hvec[4].iov_len = http_sc->message_size;
-					found = 1;
-                        		break;
-                		}
-        		}
-		
-			if (!found) {
-				uwsgi_log("invalid Pump response (status code).\n"); 
-				goto clear; 
+			if (uwsgi_response_prepare_headers(wsgi_req, sc, 3)) {
+				uwsgi_log("unable to prepare response headers\n");
 			}
 
-			wsgi_req->hvec[0].iov_base = wsgi_req->protocol;
-			wsgi_req->hvec[0].iov_len = wsgi_req->protocol_len;
-			wsgi_req->hvec[1].iov_base = " ";
-			wsgi_req->hvec[1].iov_len = 1;
-			wsgi_req->hvec[2].iov_base = sc;
-			wsgi_req->hvec[2].iov_len = 3;
-			wsgi_req->hvec[3].iov_base = " ";
-			wsgi_req->hvec[3].iov_len = 1;
-			wsgi_req->hvec[5].iov_base = "\r\n";
-			wsgi_req->hvec[5].iov_len = 2;
-
-			UWSGI_RELEASE_GIL
-                	wsize = wsgi_req->socket->proto_writev_header(wsgi_req, wsgi_req->hvec, 6);
-                	UWSGI_GET_GIL
-                	if (wsize < 0) {
-                        	uwsgi_error("writev()");
-                	}
-                	wsgi_req->headers_size += wsize;
-
 			PyObject *hhkey, *hhvalue;
 #ifdef UWSGI_PYTHON_OLD
 			int hhpos = 0;
@@ -263,53 +207,32 @@ int uwsgi_response_subhandler_pump(struct wsgi_request *wsgi_req) {
 			while (PyDict_Next(headers, &hhpos, &hhkey, &hhvalue)) {
 				if (!PyString_Check(hhkey)) continue;
 
-				wsgi_req->hvec[0].iov_base = PyString_AsString(hhkey);
-				wsgi_req->hvec[0].iov_len = PyString_Size(hhkey);	
-				((char*)wsgi_req->hvec[0].iov_base)[0] = toupper((int) ((char*)wsgi_req->hvec[0].iov_base)[0]);
+				char *k = PyString_AsString(hhkey);
+				size_t kl = PyString_Size(hhkey);	
+				k[0] = toupper((int) k[0]);
 
-				wsgi_req->hvec[1].iov_base = ": ";
-                        	wsgi_req->hvec[1].iov_len = 2;
-
-				wsgi_req->hvec[3].iov_base = "\r\n";
-                        	wsgi_req->hvec[3].iov_len = 2;
 				if (PyList_Check(hhvalue)) {
 					for(i=0;i<PyList_Size(hhvalue);i++) {
 						PyObject *item = PyList_GetItem(hhvalue, i);
 						if (PyString_Check(item)) {
-							wsgi_req->hvec[2].iov_base = PyString_AsString(item);
-                                        		wsgi_req->hvec[2].iov_len = PyString_Size(item);
-                                        		UWSGI_RELEASE_GIL
-                                        		wsize = wsgi_req->socket->proto_writev_header(wsgi_req, wsgi_req->hvec, 4);
-                                        		UWSGI_GET_GIL
-                                        		if (wsize < 0) {
-                                                		uwsgi_error("writev()");
-                                        		}
-                                        		wsgi_req->headers_size += wsize;
+							if (uwsgi_response_add_header(wsgi_req, k, kl, PyString_AsString(item), PyString_Size(item))) goto clear;
 						}
 					}	
 				}
 				else if (PyString_Check(hhvalue)) {
-					wsgi_req->hvec[2].iov_base = PyString_AsString(hhvalue);
-                                	wsgi_req->hvec[2].iov_len = PyString_Size(hhvalue);
-					UWSGI_RELEASE_GIL
-                        		wsize = wsgi_req->socket->proto_writev_header(wsgi_req, wsgi_req->hvec, 4);
-                        		UWSGI_GET_GIL
-                        		if (wsize < 0) {
-                                		uwsgi_error("writev()");
-                        		}
-                        		wsgi_req->headers_size += wsize;
+					if (uwsgi_response_add_header(wsgi_req, k, kl, PyString_AsString(hhvalue), PyString_Size(hhvalue))) goto clear;
 				}
 			}
 
-			wsgi_req->socket->proto_write(wsgi_req, "\r\n", 2);
 			Py_INCREF((PyObject *)wsgi_req->async_placeholder);
 
 			if (PyString_Check((PyObject *)wsgi_req->async_placeholder)) {
-                		if ((wsize = wsgi_req->socket->proto_write(wsgi_req, PyString_AsString(wsgi_req->async_placeholder), PyString_Size(wsgi_req->async_placeholder))) < 0) {
-                        		uwsgi_error("write()");
-                        		goto clear;
-                		}
-                		wsgi_req->response_size += wsize;
+				UWSGI_RELEASE_GIL
+				uwsgi_response_write_body_do(wsgi_req, PyString_AsString(wsgi_req->async_placeholder), PyString_Size(wsgi_req->async_placeholder));
+				UWSGI_GET_GIL
+				uwsgi_py_check_write_errors {
+                                        uwsgi_py_write_exception(wsgi_req);
+                                }
                 		goto clear;
         		}
 #ifdef PYTHREE
@@ -318,11 +241,12 @@ int uwsgi_response_subhandler_pump(struct wsgi_request *wsgi_req) {
 			else if (PyFile_Check((PyObject *)wsgi_req->async_placeholder)) {
 				wsgi_req->sendfile_fd = fileno(PyFile_AsFile((PyObject *)wsgi_req->async_placeholder));
 #endif
-                		wsize = uwsgi_sendfile(wsgi_req);
-				if (wsize < 0) {
-					goto clear;
-				}
-                		wsgi_req->response_size += wsize;
+				UWSGI_RELEASE_GIL
+                		uwsgi_response_sendfile_do(wsgi_req, wsgi_req->sendfile_fd, 0, 0);
+                		UWSGI_GET_GIL
+				uwsgi_py_check_write_errors {
+                                        uwsgi_py_write_exception(wsgi_req);
+                                }
 				goto clear;
 			}
 
@@ -335,11 +259,11 @@ int uwsgi_response_subhandler_pump(struct wsgi_request *wsgi_req) {
 			if (!wsgi_req->async_placeholder) {
 				goto clear;
 			}
-#ifdef UWSGI_ASYNC
+
 			if (uwsgi.async > 1) {
 				return UWSGI_AGAIN;
 			}
-#endif
+
 		}
 		else {
 			uwsgi_log("invalid Pump response.\n"); 
@@ -352,18 +276,22 @@ int uwsgi_response_subhandler_pump(struct wsgi_request *wsgi_req) {
 	pychunk = PyIter_Next(wsgi_req->async_placeholder);
 
 	if (!pychunk) {
-		if (PyErr_Occurred()) PyErr_Print();
+		if (PyErr_Occurred()) {
+			uwsgi_manage_exception(wsgi_req, uwsgi.catch_exceptions);
+		}
 		goto clear;
 	}
 
 
 	if (PyString_Check(pychunk)) {
-		if ((wsize = wsgi_req->socket->proto_write(wsgi_req,  PyString_AsString(pychunk), PyString_Size(pychunk))) < 0) {
-			uwsgi_error("write()");
+		UWSGI_RELEASE_GIL
+		uwsgi_response_write_body_do(wsgi_req, PyString_AsString(pychunk), PyString_Size(pychunk));
+		UWSGI_GET_GIL
+		uwsgi_py_check_write_errors {
+                	uwsgi_py_write_exception(wsgi_req);
 			Py_DECREF(pychunk);
 			goto clear;
-		}
-		wsgi_req->response_size += wsize;
+                }
 	}
 
 
